@@ -14,6 +14,7 @@ except ImportError:
         TerminalSize = namedtuple('TerminalSize', ['columns', 'lines'])
         return TerminalSize(int(os.environ.get('COLUMNS', fallback[0])), int(os.environ.get('LINES', fallback[1])))
 import sys
+import platform
 import time
 import unicodedata
 
@@ -434,27 +435,25 @@ def dir_is_cachedir(path):
 
 def dir_is_tagged(path, exclude_caches, exclude_if_present):
     """Determines whether the specified path is excluded by being a cache
-    directory or containing the user-specified tag file. Returns the
-    path of the tag file (either CACHEDIR.TAG or the matching
-    user-specified file)
+    directory or containing user-specified tag files. Returns a list of the
+    paths of the tag files (either CACHEDIR.TAG or the matching
+    user-specified files).
     """
+    tag_paths = []
     if exclude_caches and dir_is_cachedir(path):
-        return os.path.join(path, 'CACHEDIR.TAG')
+        tag_paths.append(os.path.join(path, 'CACHEDIR.TAG'))
     if exclude_if_present is not None:
         for tag in exclude_if_present:
             tag_path = os.path.join(path, tag)
             if os.path.isfile(tag_path):
-                return tag_path
-    return None
+                tag_paths.append(tag_path)
+    return tag_paths
 
 
 def format_time(t):
-    """Format datetime suitable for fixed length list output
+    """use ISO-8601 date and time format
     """
-    if abs((datetime.now() - t).days) < 365:
-        return t.strftime('%b %d %H:%M')
-    else:
-        return t.strftime('%b %d  %Y')
+    return t.strftime('%Y-%m-%d %H:%M:%S')
 
 
 def format_timedelta(td):
@@ -510,7 +509,7 @@ def sizeof_fmt_decimal(num, suffix='B', sep='', precision=2):
 
 
 def format_archive(archive):
-    return '%-36s %s' % (archive.name, to_localtime(archive.ts).strftime('%c'))
+    return '%-36s %s' % (archive.name, format_time(to_localtime(archive.ts)))
 
 
 def memoize(function):
@@ -698,19 +697,6 @@ def location_validator(archive=None):
     return validator
 
 
-def read_msgpack(filename):
-    with open(filename, 'rb') as fd:
-        return msgpack.unpack(fd)
-
-
-def write_msgpack(filename, d):
-    with open(filename + '.tmp', 'wb') as fd:
-        msgpack.pack(d, fd)
-        fd.flush()
-        os.fsync(fd.fileno())
-    os.rename(filename + '.tmp', filename)
-
-
 def decode_dict(d, keys, encoding='utf-8', errors='surrogateescape'):
     for key in keys:
         if isinstance(d.get(key), bytes):
@@ -889,3 +875,90 @@ def yes(msg=None, retry_msg=None, false_msg=None, true_msg=None,
         if retry_msg:
             print(retry_msg, file=ofile, end='')
             ofile.flush()
+
+
+class ProgressIndicatorPercent:
+    def __init__(self, total, step=5, start=0, same_line=False, msg="%3.0f%%", file=sys.stderr):
+        """
+        Percentage-based progress indicator
+
+        :param total: total amount of items
+        :param step: step size in percent
+        :param start: at which percent value to start
+        :param same_line: if True, emit output always on same line
+        :param msg: output message, must contain one %f placeholder for the percentage
+        :param file: output file, default: sys.stderr
+        """
+        self.counter = 0  # 0 .. (total-1)
+        self.total = total
+        self.trigger_at = start  # output next percentage value when reaching (at least) this
+        self.step = step
+        self.file = file
+        self.msg = msg
+        self.same_line = same_line
+
+    def progress(self, current=None):
+        if current is not None:
+            self.counter = current
+        pct = self.counter * 100 / self.total
+        self.counter += 1
+        if pct >= self.trigger_at:
+            self.trigger_at += self.step
+            return pct
+
+    def show(self, current=None):
+        pct = self.progress(current)
+        if pct is not None:
+            return self.output(pct)
+
+    def output(self, percent):
+        print(self.msg % percent, file=self.file, end='\r' if self.same_line else '\n')  # python 3.3 gives us flush=True
+        self.file.flush()
+
+    def finish(self):
+        if self.same_line:
+            print(" " * len(self.msg % 100.0), file=self.file, end='\r')
+
+
+
+class ProgressIndicatorEndless:
+    def __init__(self, step=10, file=sys.stderr):
+        """
+        Progress indicator (long row of dots)
+
+        :param step: every Nth call, call the func
+        :param file: output file, default: sys.stderr
+        """
+        self.counter = 0  # call counter
+        self.triggered = 0  # increases 1 per trigger event
+        self.step = step  # trigger every <step> calls
+        self.file = file
+
+    def progress(self):
+        self.counter += 1
+        trigger = self.counter % self.step == 0
+        if trigger:
+            self.triggered += 1
+        return trigger
+
+    def show(self):
+        trigger = self.progress()
+        if trigger:
+            return self.output(self.triggered)
+
+    def output(self, triggered):
+        print('.', end='', file=self.file)  # python 3.3 gives us flush=True
+        self.file.flush()
+
+    def finish(self):
+        print(file=self.file)
+
+
+def sysinfo():
+    info = []
+    info.append('Platform: %s' % (' '.join(platform.uname()), ))
+    if sys.platform.startswith('linux'):
+        info.append('Linux: %s %s %s  LibC: %s %s' % (platform.linux_distribution() + platform.libc_ver()))
+    info.append('Python: %s %s' % (platform.python_implementation(), platform.python_version()))
+    info.append('')
+    return '\n'.join(info)
